/*
 * File:     PrintPreviewCA.java
 * Project:  MPI Linguistic Application
 * Date:     31 October 2005
 *
 * Copyright (C) 2001-2005  Max Planck Institute for Psycholinguistics
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package mpi.eudico.client.annotator.commands;

import mpi.eudico.client.annotator.ElanLocale;
import mpi.eudico.client.annotator.ViewerManager2;


/**
 * A CommandAction to generate printout.
 *
 * @author Han Sloetjes
 */
public class PrintPreviewCA extends CommandAction {
    /**
     * Creates a new PrintPreviewCA instance
     *
     * @param viewerManager the viewer manager
     */
    public PrintPreviewCA(ViewerManager2 viewerManager) {
        super(viewerManager, ELANCommandFactory.PREVIEW);
        putValue(SHORT_DESCRIPTION,
            ElanLocale.getString(ELANCommandFactory.PREVIEW + "ToolTip"));
        putValue(DEFAULT, "PREVIEW");
        updateLocale(); //because of mnemonic
    }

    /**
     * Creates a new Command
     */
    protected void newCommand() {
        command = ELANCommandFactory.createCommand(vm.getTranscription(),
                ELANCommandFactory.PREVIEW);
    }

    /**
     * The receiver is a Transcription.
     *
     * @return a Transcription object
     */
    protected Object getReceiver() {
        return vm.getTranscription();
    }

    /**
     * A Selection object.
     *
     * @return the selection
     */
    protected Object[] getArguments() {
        return new Object[] { vm.getSelection() };
    }
}
