/*
 * File:     ElanMatch.java
 * Project:  MPI Linguistic Application
 * Date:     31 October 2005
 *
 * Copyright (C) 2001-2005  Max Planck Institute for Psycholinguistics
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package mpi.eudico.client.annotator.search.result.model;

import mpi.eudico.server.corpora.clom.Annotation;

import mpi.search.content.result.model.AbstractContentMatch;


/**
 * Created on Jul 22, 2004
 * @author Alexander Klassmann
 * @version Jul 22, 2004
 */
public class ElanMatch extends AbstractContentMatch {
    /** Holds value of property DOCUMENT ME! */
    final private Annotation annotation;

    /**
     * Creates a new ElanMatch instance
     *
     * @param annotation DOCUMENT ME!
     */
    public ElanMatch(Annotation annotation) {
        this.annotation = annotation;
    }

    /**
     * Creates a new ElanMatch instance
     *
     * @param annotation DOCUMENT ME!
     * @param index DOCUMENT ME!
     */
    public ElanMatch(Annotation annotation, int index) {
        this(annotation);
        setIndex(index);
    }

    /**
     * Creates a new ElanMatch instance
     *
     * @param annotation DOCUMENT ME!
     * @param $paramType$ DOCUMENT ME!
     * @param substringIndices DOCUMENT ME!
     */
    public ElanMatch(Annotation annotation, int index, int[][] substringIndices) {
        this(annotation, index);
        setMatchedSubstringIndices(substringIndices);
    }

    /**
     * Creates a new ElanMatch instance
     *
     * @param annotation DOCUMENT ME!
     * @param index DOCUMENT ME!
     * @param leftContext DOCUMENT ME!
     * @param rightContext DOCUMENT ME!
     * @param substringIndices DOCUMENT ME!
     */
    public ElanMatch(Annotation annotation, int index, String leftContext,
        String rightContext, int[][] substringIndices) {
        this(annotation, index);
        setLeftContext(leftContext);
        setRightContext(rightContext);
        setMatchedSubstringIndices(substringIndices);
    }

    /**
     * DOCUMENT ME!
     *
     * @param fileName DOCUMENT ME!
     */
    public void setFileName(String fileName) {
        this.fileName = fileName;
    }

    /* (non-Javadoc)
     * @see mpi.eudico.search.advanced.result.model.Result#getTierName()
     */
    public String getTierName() {
        String name = "";

        try {
            name = annotation.getTier().getName();
        } catch (Exception e) {
            e.printStackTrace();
        }

        return name;
    }

    /* (non-Javadoc)
     * @see mpi.eudico.search.advanced.result.model.Result#getValue()
     */
    public String getValue() {
        return annotation.getValue();
    }

    /* (non-Javadoc)
     * @see mpi.eudico.search.advanced.result.model.Result#getBeginTime()
     */
    public long getBeginTimeBoundary() {
        return annotation.getBeginTimeBoundary();
    }

    /* (non-Javadoc)
     * @see mpi.eudico.search.advanced.result.model.Result#getEndTime()
     */
    public long getEndTimeBoundary() {
        return annotation.getEndTimeBoundary();
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public Annotation getAnnotation() {
        return annotation;
    }

    /**
     * DOCUMENT ME!
     *
     * @param context DOCUMENT ME!
     */
    public void setLeftContext(String context) {
        leftContext = context;
    }

    /**
     * DOCUMENT ME!
     *
     * @param context DOCUMENT ME!
     */
    public void setRightContext(String context) {
        rightContext = context;
    }

    /**
     * DOCUMENT ME!
     *
     * @param index DOCUMENT ME!
     */
    public void setIndex(int index) {
        this.index = index;
    }

    /**
     * DOCUMENT ME!
     *
     * @param substringIndices DOCUMENT ME!
     */
    public void setMatchedSubstringIndices(int[][] substringIndices) {
        this.matchedSubstringIndices = substringIndices;
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public String getId() {
        return null;
    }
}
