/*
 * File:     AnnotationCore.java
 * Project:  MPI Linguistic Application
 * Date:     31 October 2005
 *
 * Copyright (C) 2001-2005  Max Planck Institute for Psycholinguistics
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package mpi.eudico.server.corpora.clom;


/**
 * Search results may arise from outside the ACM model.
 * For this purpose it is important to have a "minimalist" annotation,
 * containing only time information and value, but no hierachical structure
 *
 * Created on Jul 23, 2004
 * @author Alexander Klassmann
 * @version Jul 23, 2004
 */
public interface AnnotationCore {
    /**
     * <p>MK:02/06/18<br>
     * Only subclass AlignableAnnotation  (ALA) has a getBegin() method,
     * which returns the begin time in milliseconds.
     * RefAnnotions (REA) don't have a begin time.
     * REA still need something like
     * a begin time, because they have to drawn somewhere on screen.
     * This time is not the begin time but the begin time boundary.
     * Note 1:<br>
     * You cannot always use the begin time
     * of the parent/root AlignableAnnotation. Proof: two
     * REA have the same parent ALA,
     * the second REA starts in the middle of the ALA. <br>
     * Note 2:<br>
     * For an ALA, which begin timeslot is timealigned,
     * getBegin() and getBeginTimeBoundary() are identical.
     * </p>
     *
     * */
    public long getBeginTimeBoundary();

    /**
     * <p>MK:02/06/18<br>
     * see getBeginTimeBoundary().
     * </p>
     *
     * */
    public long getEndTimeBoundary();

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public String getValue();
}
