/*
 * File:     ToolboxEncoderInfo.java
 * Project:  MPI Linguistic Application
 * Date:     31 October 2005
 *
 * Copyright (C) 2001-2005  Max Planck Institute for Psycholinguistics
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package mpi.eudico.server.corpora.clomimpl.shoebox;

import mpi.eudico.server.corpora.clom.EncoderInfo;

import mpi.eudico.server.corpora.clomimpl.shoebox.interlinear.Interlinearizer;

import java.util.List;


/**
 * Toolbox specific EncoderInfo implementation
 * may 06 (HS): added boolean for two export options:<br>
 * - whether or not to recalculate begin and end times of annotations based on the master media offset<br>
 *
 * @author hennie
 */
public class ToolboxEncoderInfo implements EncoderInfo {
    /** Holds value of property DOCUMENT ME! */
    public static final int TYPFILE = 0;

    /** Holds value of property DOCUMENT ME! */
    public static final int DEFINED_MARKERS = 1;

    /** Holds value of property DOCUMENT ME! */
    public static final int TIERNAMES = 2;
    private int pageWidth;
    private int markerSource;
    private int timeFormat;

    // may 2006: add db type to the encoder object instead of a static field in ShoeboxTypFile
    private String databaseType = "";

    // may 2006: add markers to the encoder object instead of a static field in ShoeboxTypFile
    private List markers;
    private boolean correctAnnotationTimes = true;

    /**
     * Creates a new ToolboxEncoderInfo instance
     *
     * @param pageWidth DOCUMENT ME!
     * @param markerSource DOCUMENT ME!
     */
    public ToolboxEncoderInfo(int pageWidth, int markerSource) {
        this(pageWidth, markerSource, Interlinearizer.SSMS);
    }

    /**
     * Creates a new ToolboxEncoderInfo instance
     *
     * @param pageWidth DOCUMENT ME!
     * @param markerSource DOCUMENT ME!
     * @param timeFormat DOCUMENT ME!
     */
    public ToolboxEncoderInfo(int pageWidth, int markerSource, int timeFormat) {
        this.pageWidth = pageWidth;
        this.markerSource = markerSource;
        this.timeFormat = timeFormat;
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public int getPageWidth() {
        return pageWidth;
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public int getMarkerSource() {
        return markerSource;
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public int getTimeFormat() {
        return timeFormat;
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public boolean getCorrectAnnotationTimes() {
        return correctAnnotationTimes;
    }

    /**
     * DOCUMENT ME!
     *
     * @param correctAnnotationTimes DOCUMENT ME!
     */
    public void setCorrectAnnotationTimes(boolean correctAnnotationTimes) {
        this.correctAnnotationTimes = correctAnnotationTimes;
    }

    /**
     * @return Returns the databaseType.
     */
    public String getDatabaseType() {
        return databaseType;
    }

    /**
     * @param databaseType The databaseType to set.
     */
    public void setDatabaseType(String databaseType) {
        this.databaseType = databaseType;
    }

    /**
     * @return Returns the markers.
     */
    public List getMarkers() {
        return markers;
    }

    /**
     * @param markers The markers to set.
     */
    public void setMarkers(List markers) {
        this.markers = markers;
    }
}
