/*
 * File:     CVentry.java
 * Project:  MPI Linguistic Application
 * Date:     31 October 2005
 *
 * Copyright (C) 2001-2005  Max Planck Institute for Psycholinguistics
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package mpi.util;

import java.io.Serializable;


/**
 * class CVentry implements the functions  of a Controlled Vocabulary item
 * there is a tag to show in a menu, a value and a description
 */
public class CVentry implements DescriptedObject, Serializable {
    /** Holds value of property DOCUMENT ME! */
    private final String value;

    /** Holds value of property DOCUMENT ME! */
    private final String tag;

    /** Holds value of property DOCUMENT ME! */
    private final String followup;

    /** Holds value of property DOCUMENT ME! */
    private final String description;

    /** Holds the number of occurrences, if CVentry is created from a data base */
    private final int numberOfOccurrences;

    /**
     * Creates a new CVentry instance
     *
     * @param tag DOCUMENT ME!
     * @param value DOCUMENT ME!
     * @param followup DOCUMENT ME!
     * @param description DOCUMENT ME!
     */
    public CVentry(String tag, String value, String followup, String description) {
        this(tag, value, followup, description, -1);
    }

    /**
     * Creates a new CVentry instance
     *
     * @param tag DOCUMENT ME!
     * @param value DOCUMENT ME!
     * @param followup DOCUMENT ME!
     * @param description DOCUMENT ME!
     */
    public CVentry(String tag, String value, String followup,
        String description, int numberOfOccurrences) {
        this.value = value;
        this.followup = followup;
        this.description = description;
        this.numberOfOccurrences = numberOfOccurrences;

        if (tag != null) {
            this.tag = tag;
        } else {
            int start = value.lastIndexOf('/') + 1;
            int end = value.lastIndexOf(".xml");

            if (end > 0) {
                this.tag = value.substring(start, end);
            } else {
                this.tag = value;
            }
        }
    }

    /**
     * Creates a new CVentry instance
     *
     * @param value
     * @param description
     */
    public CVentry(String value, String description) {
        this("", value, "", description);
    }

    /**
     * Creates a new CVentry instance
     *
     * @param value DOCUMENT ME!
     * @param description DOCUMENT ME!
     */
    public CVentry(String value, String description, int numberOfOccurrences) {
        this("", value, "", description, numberOfOccurrences);
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public String getValue() {
        return value;
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public String getName() {
        return tag;
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public String getDescription() {
        return description;
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public String getFollowUp() {
        return followup;
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public String toString() {
        return tag;
    }

    /**
     * returns number of occurrences of a specific item within a data base
     * @return
     */
    public int getNumberOfOccurrences() {
        return numberOfOccurrences;
    }
}
